use std::{
    net::TcpListener,
    sync::{
        atomic::{self, AtomicBool},
        Arc,
    },
    thread::available_parallelism,
};

use crate::{handlers::handlers::handle_connection, routing::routes::Route, threading::ThreadPool};

pub struct Config<'a> {
    mountpoints: Option<Vec<&'static str>>,
    address: TcpListener,
    threadpool: ThreadPool,
    routes: Option<Vec<Route<'a>>>,
}

impl<'a> Config<'a> {
    pub fn mount(self, mountpoint: &'static str, routes: Vec<Route<'static>>) -> Config<'a> {
        if self.mountpoints == None {
            return Config {
                mountpoints: Some(vec![mountpoint]),
                routes: Some(routes),
                ..self
            };
        } else {
            return self;
        }
    }
    pub fn launch(self) {
        let running = Arc::new(AtomicBool::new(true));

        // Clone a reference to `running` to pass to the signal handler closure
        let running_for_handler = running.clone();

        // Set up a signal handler for SIGINT
        ctrlc::set_handler(move || {
            // When the user sends a SIGINT signal (e.g. by pressing CTRL+C),
            // set the `running` flag to false to initiate a graceful shutdown
            println!("SIGNIT received, shutting down gracefully");
            running_for_handler.store(false, atomic::Ordering::SeqCst);
        })
        .expect("Error setting Ctrl-C handler");

        for stream in self.address.incoming() {
            if !running.load(atomic::Ordering::SeqCst) {
                break;
            }
            let stream = stream.unwrap();
            let mountpoint = self.mountpoints.clone().unwrap();
            self.threadpool
                .execute(move || handle_connection(stream, &mountpoint[0].to_string()))
        }
    }
}
pub fn build(ip: &str) -> Config {
    let listener = TcpListener::bind(ip).unwrap();
    let ip = ip.splitn(2, ":").collect::<Vec<&str>>();
    if ip.len() != 2 {
        panic!("Invalid IP Address");
    }
    let port = ip[1];
    let ip = ip[0];
    let workers = available_parallelism().unwrap().get();
    let threadpool = ThreadPool::new(workers);
    println!(
        "\x1b[34m⚙ Configuration\x1b[0m
  >> \x1b[34mIp\x1b[0m: {ip}
  >> \x1b[34mPort\x1b[0m: {port}
  >> \x1b[34mWorkers\x1b[0m: {workers}
\n
Server has launched from {ip}:{port}.
"
    );
    Config {
        mountpoints: None,
        routes: None,
        address: listener,
        threadpool,
    }
}
